'''
Created on Jul 15, 2016

@author: feliu

Copyright (c) 2022 by Cisco Systems, Inc.
All rights reserved.

Common utility function(s)
'''

import urllib.request, urllib.error, urllib.parse
import ssl
import calendar
import json
import socket
import struct

def access_url_once(url, headers, payload):
    print("payload=", payload)
    print("url=", url)
    req = urllib.request.Request(url)
    if 'Content-Type' in headers:
        req.add_header('Content-Type', headers['Content-Type'])
    if 'cookie' in headers:
        req.add_header('cookie', headers['cookie'])
    # custom_context is needed for some ssl version to avoid certificate check error
    custom_context = ssl.SSLContext(ssl.PROTOCOL_TLSv1) if hasattr(ssl, 'SSLContext') else None
    print("custom_context=", custom_context)
    resp = None
    if custom_context:
        try:
            resp = urllib.request.urlopen(req, context=custom_context, timeout=5) if payload is None else urllib.request.urlopen(req, payload, context=custom_context, timeout=5)
        except urllib.error.URLError:
            # if TLSv1 is not supported, try SSLv23
            custom_context = ssl.SSLContext(ssl.PROTOCOL_SSLv23) if hasattr(ssl, 'SSLContext') else None
            resp = urllib.request.urlopen(req, context=custom_context, timeout=5) if payload is None else urllib.request.urlopen(req, payload, context=custom_context, timeout=5)
    else:
        resp = urllib.request.urlopen(req, timeout=5) if payload is None else urllib.request.urlopen(req, payload, timeout=5)
    return resp

def load_cookies(cookie_file):
    try:
      f = open(cookie_file, "r")
      cookies = json.load(f)
      f.close()
      return cookies
    except (IOError, ValueError):
      # cookie_file not exist yet, no cookie
      return {}

def get_cookie(key, cookie_file):
    ''' get cookie based on the key (which is combination of APIC IP and login_username). '''
    cookies = load_cookies(cookie_file)
    if len(cookies) == 0:
        return None
    return cookies.get(key, None)

def update_cookies(cookies, cookie_file):
    f = open(cookie_file, "w")
    json.dump(cookies, f)
    f.close()

